@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtkalignment.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
@@ -41,9 +41,7 @@ enum {
   PROP_TOP_PADDING,
   PROP_BOTTOM_PADDING,
   PROP_LEFT_PADDING,
-  PROP_RIGHT_PADDING,
-  
-  PROP_LAST
+  PROP_RIGHT_PADDING
 };
 
 #define GTK_ALIGNMENT_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_ALIGNMENT, GtkAlignmentPrivate))
@@ -201,7 +199,7 @@ gtk_alignment_init (GtkAlignment *alignment)
 {
   GtkAlignmentPrivate *priv;
   
-  GTK_WIDGET_SET_FLAGS (alignment, GTK_NO_WINDOW);
+  gtk_widget_set_has_window (GTK_WIDGET (alignment), FALSE);
   gtk_widget_set_redraw_on_allocate (GTK_WIDGET (alignment), FALSE);
 
   alignment->xalign = 0.5;
@@ -423,7 +421,7 @@ gtk_alignment_size_request (GtkWidget      *widget,
   requisition->width = GTK_CONTAINER (widget)->border_width * 2;
   requisition->height = GTK_CONTAINER (widget)->border_width * 2;
 
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     {
       GtkRequisition child_requisition;
       
@@ -458,7 +456,7 @@ gtk_alignment_size_allocate (GtkWidget     *widget,
   alignment = GTK_ALIGNMENT (widget);
   bin = GTK_BIN (widget);
   
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     {
       gtk_widget_get_child_requisition (bin->child, &child_requisition);
 
@@ -468,8 +466,8 @@ gtk_alignment_size_allocate (GtkWidget     *widget,
       padding_horizontal = priv->padding_left + priv->padding_right;
       padding_vertical = priv->padding_top + priv->padding_bottom;
 
-      width = allocation->width - padding_horizontal - 2 * border_width;
-      height = allocation->height - padding_vertical - 2 * border_width;
+      width  = MAX (1, allocation->width - padding_horizontal - 2 * border_width);
+      height = MAX (1, allocation->height - padding_vertical - 2 * border_width);
     
       if (width > child_requisition.width)
 	child_allocation.width = (child_requisition.width *
@@ -559,10 +557,10 @@ gtk_alignment_set_padding (GtkAlignment    *alignment,
 /**
  * gtk_alignment_get_padding:
  * @alignment: a #GtkAlignment
- * @padding_top: location to store the padding for the top of the widget, or %NULL
- * @padding_bottom: location to store the padding for the bottom of the widget, or %NULL
- * @padding_left: location to store the padding for the left of the widget, or %NULL
- * @padding_right: location to store the padding for the right of the widget, or %NULL
+ * @padding_top: (allow-none): location to store the padding for the top of the widget, or %NULL
+ * @padding_bottom: (allow-none): location to store the padding for the bottom of the widget, or %NULL
+ * @padding_left: (allow-none): location to store the padding for the left of the widget, or %NULL
+ * @padding_right: (allow-none): location to store the padding for the right of the widget, or %NULL
  *
  * Gets the padding on the different sides of the widget.
  * See gtk_alignment_set_padding ().
