@@ -63,7 +63,7 @@ gtk_pixmap_class_init (GtkPixmapClass *class)
 static void
 gtk_pixmap_init (GtkPixmap *pixmap)
 {
-  GTK_WIDGET_SET_FLAGS (pixmap, GTK_NO_WINDOW);
+  gtk_widget_set_has_window (GTK_WIDGET (pixmap), FALSE);
 
   pixmap->pixmap = NULL;
   pixmap->mask = NULL;
@@ -135,7 +135,7 @@ gtk_pixmap_set (GtkPixmap *pixmap,
 	  GTK_WIDGET (pixmap)->requisition.width = 0;
 	  GTK_WIDGET (pixmap)->requisition.height = 0;
 	}
-      if (GTK_WIDGET_VISIBLE (pixmap))
+      if (gtk_widget_get_visible (GTK_WIDGET (pixmap)))
 	{
 	  if ((GTK_WIDGET (pixmap)->requisition.width != oldwidth) ||
 	      (GTK_WIDGET (pixmap)->requisition.height != oldheight))
@@ -201,7 +201,7 @@ gtk_pixmap_expose (GtkWidget      *widget,
 	  gdk_gc_set_clip_origin (widget->style->black_gc, x, y);
 	}
 
-      if (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE
+      if (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE
           && pixmap->build_insensitive)
         {
 	  if (!pixmap->pixmap_insensitive)
@@ -235,7 +235,7 @@ gtk_pixmap_set_build_insensitive (GtkPixmap *pixmap, gboolean build)
 
   pixmap->build_insensitive = build;
 
-  if (GTK_WIDGET_VISIBLE (pixmap))
+  if (gtk_widget_get_visible (GTK_WIDGET (pixmap)))
     {
       gtk_widget_queue_draw (GTK_WIDGET (pixmap));
     }
